@@ -1,17 +1,24 @@
 #-*- coding:utf-8 -*-import time
 import re, pdb, time
 
+import django.dispatch
 from django.core.urlresolvers import resolve
 from django.conf import settings
 from django.utils.cache import patch_vary_headers
 from django.utils.http import cookie_date
 from django.utils.importlib import import_module
-from django.http  import  HttpResponseRedirect
+from django.http  import  HttpResponseRedirect, HttpResponse
 from django.contrib.sessions.middleware import SessionMiddleware
 # Obscure the session id when passing it around in HTML
 from cookieless.utils import CryptSession
 from cookieless.config import LINKS_RE, DEFAULT_SETTINGS
 
+# Add a signal as a hook for the creation or saving of cookieless sessions
+# since these may need different handling to normal cookie based ones
+# NB: There is the django.contrib.sessions.models.Session model to hook to, 
+#     but creates and saves happen later and for both cookie and cookieless sessions
+cookieless_signal = django.dispatch.Signal() 
+
 class CookielessSessionMiddleware(object):
     """ Django snippets julio carlos and Ivscar 
         http://djangosnippets.org/snippets/1540/
@@ -22,6 +29,7 @@ class CookielessSessionMiddleware(object):
         with 'cookieless.middleware.CookielessSessionMiddleware'
 
         NB: Remember only decorated methods are cookieless
+        cookieless sessions get the no_cookies = True key added
     """
 
     def __init__(self):
@@ -49,10 +57,13 @@ class CookielessSessionMiddleware(object):
         name = settings.SESSION_COOKIE_NAME
         session_key = ''
         match = resolve(request.path)
-
+        no_cookies = False
+        
         if match and getattr(match.func, 'no_cookies', False):
-            session_key = self._sesh.decrypt(request, 
-                                             request.POST.get(name, None))
+            no_cookies = True
+            if request.POST:
+                session_key = self._sesh.decrypt(request, 
+                                                 request.POST.get(name, None))
             if not session_key and self.settings.get('USE_GET', False):
                 session_key = self._sesh.decrypt(request, 
                                                  request.GET.get(name, ''))
@@ -72,8 +83,14 @@ class CookielessSessionMiddleware(object):
         # If the session_key isn't tied to a session - create a new one
         if not session_key:
             request.session = self.engine.SessionStore() 
+            if no_cookies:
+                request.session['no_cookies'] = True
+                # Flag it here so we can send created session signal 
+                # with data later on in process_response
+                request.session['created_cookieless'] = True
             request.session.save()
 
+
     def process_response(self, request, response):
         """
         Copied from contrib.session.middleware with no_cookies switch added ...
@@ -89,7 +106,9 @@ class CookielessSessionMiddleware(object):
                     # - may be attached to a user - so always start a new separate one
                     cookie_key = request.COOKIES.get(settings.SESSION_COOKIE_NAME, '')
                     if cookie_key == request.session.session_key:
-                        request.session = self.engine.SessionStore() 
+                        request.session = self.engine.SessionStore()
+                        request.session['no_cookies'] = True
+                        request.session['created_cookieless'] = True
                         request.session.save()
                 if self.settings.get('DELETE_COOKIES', False):
                     # Blat any existing cookies
@@ -97,12 +116,17 @@ class CookielessSessionMiddleware(object):
                         response.delete_cookie(key)
 
             # Dont set any new cookies
-            response.cookies.clear()
+            if hasattr(response, 'cookies'):
+                response.cookies.clear()
 
             # cookieless - do same as standard process response
             #              but dont set the cookie
             if self.settings.get('REWRITE', False):
                 response = self.nocookies_response(request, response)
+            # Check for created flag for signal and turn off
+            created = request.session.get('created_cookieless', False)
+            if created:
+                request.session['created_cookieless'] = False
             try:
                 accessed = request.session.accessed
                 modified = request.session.modified
@@ -117,8 +141,9 @@ class CookielessSessionMiddleware(object):
                         max_age = request.session.get_expiry_age()
                         expires_time = time.time() + max_age
                         expires = cookie_date(expires_time)
-                # Save the session data and refresh the client cookie.
-                request.session.save()
+                    # Save the session data 
+                    request.session.save()
+                    cookieless_signal.send(sender=request, created=created)
             return response
         else:
             return self.standard_session.process_response(request, response)
@@ -161,14 +186,15 @@ class CookielessSessionMiddleware(object):
                 except:
                     pass
 
-            repl_form = '''<div><input type="hidden" name="%s" value="%s" />
-                           </div></form>'''
-            repl_form = repl_form % (name, session_key)
-
-            try:
-                response.content = self._re_forms.sub(repl_form, response.content)
-            except:
-                pass
+            # Check in case response has already got a manual session_id inserted
+            repl_form = '<input type="hidden" name="%s"' % name
+            if hasattr(response, 'content') and response.content.find(repl_form) == -1:
+                repl_form = '''%s value="%s" />
+                               </form>''' % (repl_form, session_key)
+                try:
+                    response.content = self._re_forms.sub(repl_form, response.content)
+                except:
+                    pass
             return response
         else:
             return response        
